const oracledb = require('oracledb')
const variables = require('../bin/configuration/variables')

const consulta_builder = {
    sql: {
        [variables.Module.planoFinanciamento]: `SELECT V_API_PLAN_FINANCIACION.COD_PLAN,
                                                       V_API_PLAN_FINANCIACION.DESCRIPCION,
                                                       V_API_PLAN_FINANCIACION.COD_MERCADERIA,
                                                       V_API_PLAN_FINANCIACION.COD_MONEDA
                                                FROM V_API_PLAN_FINANCIACION
                                                WHERE 1 = 1`,
        [variables.Module.planoFinanciamento + '_by_id']: `\nAND V_API_PLAN_FINANCIACION.COD_PLAN = :ID`,

        [variables.Module.mercadoria]: `SELECT V_API_MERCADERIA.COD_MERCADERIA,
                                               V_API_MERCADERIA.DESCRIPCION
                                        FROM V_API_MERCADERIA
                                        WHERE V_API_MERCADERIA.SITUACION = 'A'
                                          AND V_API_MERCADERIA.MERC_CEREAL = 'S'`,
        [variables.Module.mercadoria + '_by_id']: `\nAND V_API_MERCADERIA.COD_MERCADERIA = :ID`,

        [variables.Module.localidade]: `SELECT V_API_LOCALIDADE.COD_LOCALIDAD, V_API_LOCALIDADE.DESCRIPCION
                                        FROM V_API_LOCALIDADE
                                        WHERE 1 = 1`,
        [variables.Module.localidade + '_by_id']: `\nAND V_API_LOCALIDADE.COD_LOCALIDADE = :ID`,

        [variables.Module.filial]: `SELECT V_API_FILIAL.COD_SUCURSAL, V_API_FILIAL.DESCRIPCION
                                    FROM V_API_FILIAL
                                    WHERE 1 = 1`,
        [variables.Module.filial + '_by_id']: `\nAND V_API_FILIAL.COD_SUCURSAL = :ID`,

        [variables.Module.departamento]: `SELECT V_API_DEPARTAMENTO.COD_DEPARTAMENTO,
                                                 V_API_DEPARTAMENTO.COD_SUCURSAL,
                                                 V_API_DEPARTAMENTO.DESCRIPCION
                                          FROM V_API_DEPARTAMENTO
                                          WHERE 1 = 1`,
        [variables.Module.departamento + '_by_id']: `\nAND V_API_DEPARTAMENTO.COD_SUCURSAL = :ID`,

        [variables.Module.variedade]: `SELECT V_API_VARIEDAD.COD_VARIEDAD,
                                              V_API_VARIEDAD.COD_MERCADERIA,
                                              V_API_VARIEDAD.DESCRIPCION
                                       FROM V_API_VARIEDAD
                                       WHERE 1 = 1`,
        [variables.Module.variedade + '_by_id']: `\nAND V_API_VARIEDAD.COD_VARIEDAD = :ID`,

        [variables.Module.silo]: `SELECT V_API_SILO.COD_SILO, V_API_SILO.DESCRIPCION
                                  FROM V_API_SILO
                                  WHERE 1 = 1`,
        [variables.Module.silo + '_by_id']: `\nAND V_API_SILO.COD_SILO = :ID`,
        [variables.Module.entidade]: `select V_API_ENTIDAD.COD_ENTIDAD COD,
                                             V_API_ENTIDAD.NOMBRE      DESCRIPCION,
                                             V_API_ENTIDAD.RUC
                                      from V_API_ENTIDAD
                                      where V_API_ENTIDAD.SITUACION = 'A'
                                      order by V_API_ENTIDAD.NOMBRE`,
        [variables.Module.entidade + '_by_id']: `\nAND V_API_ENTIDAD.COD_ENTIDAD = :ID`,
        [variables.Module.schema]: `select CFGDONO.CNOMEDONO
                                    from DOLPHIN.CFGDONO
                                    order by CFGDONO.CNOMEDONO`,
        [variables.Module.processo]: `SELECT TOKEN,
                                             DECODE(STATUS, 'P', 'PROCESAMIENTO', 'F', 'TERMINADO', 'I',
                                                    'INICIADO')              STATUS,
                                             NVL(TOTAL, 0)                   TOTAL,
                                             NVL(PROCESSADO, 0)              PROCESSADO,
                                             TO_CHAR(INICIADO, 'DD/MM/YYYY') INICIADO,
                                             FINALIZADO,
                                             MENSAGEM
                                      FROM BACKUP_TABLES.ETL_PROCESSO
                                      WHERE 1 = 1`,
        [variables.Module.processo + '_by_id']: `\nAND UPPER(TOKEN) = UPPER(:ID)`,
        [variables.Module.tipoCargas]: `SELECT baltpca.ncoditpca id,
                                               baltpca.cnometpca descripcion,
                                               baltpca.censatpca entradaSaida,
                                               baltpca.ctpoptpca compraVenda,
                                               baltpca.ctpngcarg tipoNegociacao,
                                               baltpca.ncodirese codRegraApropriacao,
                                               (CASE
                                                    WHEN (baltpca.cconttpca = 'C') THEN 'Contrato'
                                                    WHEN (baltpca.cconttpca = 'D') THEN 'Despacho'
                                                    ELSE 'Normal'
                                                   END)          subsistema
                                        FROM baltpca
                                        WHERE 1 = 1`,
        [variables.Module.tipoCargas + '_by_id']: `AND baltpca.ncoditpca = :ID`,
        [variables.Module.entidadeAnalista]: `SELECT V_API_ENTIDAD.COD_ENTIDAD COD,
                                                     V_API_ENTIDAD.NOMBRE      DESCRIPCION,
                                                     V_API_ENTIDAD.RUC
                                              FROM V_API_ENTIDAD
                                              WHERE V_API_ENTIDAD.SITUACION = 'A'
                                                AND EXISTS
                                                  (SELECT NULL
                                                   FROM CFGCLEN
                                                   WHERE CFGCLEN.NCODICLAS = 5 --analista de cargas
                                                     AND CFGCLEN.NCODIENTI = V_API_ENTIDAD.COD_ENTIDAD)
                                              ORDER BY V_API_ENTIDAD.NOMBRE`,
    },
    bind_in: {
        [variables.Module.planoFinanciamento]: {COD_PLAN: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.mercadoria]: {COD_MERCADERIA: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.localidade]: {COD_LOCALIDADE: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.filial]: {COD_FILIAL: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.departamento]: {COD_DEPARTAMENTO: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.variedade]: {COD_VARIEDAD: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.silo]: {COD_SILO: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.entidade]: {COD: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
        [variables.Module.processo]: {TOKEN: {dir: oracledb.BIND_IN, type: oracledb.STRING, maxSize: 32}},
        [variables.Module.tipoCargas]: {id: {dir: oracledb.BIND_IN, type: oracledb.NUMBER}},
    }
}

function getSqlFromModulo(modulo, withId = false) {
    let sql_statement = consulta_builder.sql[modulo]
    if ((withId) && (consulta_builder.sql[modulo + '_by_id']))
        sql_statement += consulta_builder.sql[modulo + '_by_id']
    return sql_statement
}

function getBindVariablesFromModulo(modulo) {
    let bind_variable = consulta_builder.bind_in[modulo]
    return bind_variable
}

module.exports = {
    consulta_builder,
    getSqlFromModulo,
    getBindVariablesFromModulo
}
