"use strict"

const oracledb = require('oracledb')
const base = require('../bin/base/repository-base')
const consulta_builder = require('../builder/consulta-builder')

class consultaRepository extends base {

    constructor() {
        super()
    }

    async simpleExecute(statement, binds = [], opts = {}) {
        return new Promise(async (resolve, reject) => {
            let conn = await this.getConnection();

            opts.outFormat = oracledb.OBJECT;
            opts.autoCommit = false;

            try {
                const result = await conn.execute(statement, binds, opts);
                resolve(result);
            } catch (err) {
                reject(err);
            } finally {
                if (conn) {
                    try {
                        await conn.close();
                    } catch (err) {
                        console.error('error in simpleExecute ->', err);
                    }
                }
            }
        });
    }

    async find(module) {
        let sql_statement = consulta_builder.getSqlFromModulo(module)
        let retorno = await this.simpleExecute(sql_statement)
        return retorno.rows
    }

    async findById(module, id) {
        let sql_statement = consulta_builder.getSqlFromModulo(module, true)
        let bind_variables = consulta_builder.getBindVariablesFromModulo(module)
        let retorno = await this.simpleExecute(
            sql_statement,
            {ID: id},
            {bindDefs: bind_variables})
        return retorno.rows
    }
}

module.exports = consultaRepository
