'use strict'

const TokenGenerator = require('uuid-token-generator');
const repository = require("../repositories/carga-repository")
const consultaRepository = require("../repositories/consulta-repository")
const variables = require('../bin/configuration/variables')
const {RetornoModel, ExecuteError} = require('../models/retorno-model')
const Usuario = require(`../models/usuario-model`)
const tokgen = new TokenGenerator(TokenGenerator.BASE36);
const _repo = new repository()

module.exports = class CargaService {

    constructor(database_config) {
        _repo.setDatabaseConfig(database_config)
        this._consultaRepository = new consultaRepository()
        this._consultaRepository.setDatabaseConfig(database_config);
        this._usuarioLogado = new Usuario()
    }

    setUserData(carga) {
        carga.cod_sucursal = this._usuarioLogado.sucursal
        carga.cod_departamento = this._usuarioLogado.departamento
        carga.cod_usuario = this._usuarioLogado.codigo
    }

    extractDesconto(cargas, token) {
        let descontos = []

        cargas.forEach(carga => {
            carga.uuid = carga.uuid ? carga.uuid : tokgen.generate()

            this.setUserData(carga)

            if (Array.isArray(carga.descontos) && carga.descontos.length > 0) {
                carga.descontos.forEach(d => {
                    d.uuid = carga.uuid
                    d.token = token
                    descontos.push(d)
                })
            }

            delete carga.descontos
        })
        return descontos
    }

    async processoExists(token) {
        let resultado = await this._consultaRepository.findById(variables.Module.processo, token)
        return (resultado.length > 0)
    }

    checkRetorno(errorList, processo) {
        if (!processo.CODE || processo.CODE === 500) {
            errorList.push(new ExecuteError(500, processo.MSG))
            throw errorList
        }
    }

    async create(cargaDto, usuarioLogado) {
        this._usuarioLogado = usuarioLogado;
        let token_do_processo = !cargaDto.token_do_processo ? tokgen.generate() : cargaDto.token_do_processo
        let isTrade = cargaDto.istrade
        let cargas = cargaDto.cargas
        let errorList = []
        let cargas_processadas = []

        if (await this.processoExists(token_do_processo)) {
            this.checkRetorno(errorList, {MSG: `El proceso: ${token_do_processo} ya fue utilizado. Debes inicar una nueva importación.`})
        }

        let descontos = this.extractDesconto(cargas, token_do_processo)
        let _conn = await _repo.getConnection()

        try {
            let start_process = await _repo.startProcess(token_do_processo, cargas.length, isTrade, cargaDto)
            this.checkRetorno(errorList, start_process)

            let carga_process = await _repo.insert_carga(cargas, _conn)
            this.checkRetorno(errorList, carga_process)

            if (descontos.length > 0) {
                let desconto_process = await _repo.insert_descontos(descontos, _conn)
                this.checkRetorno(errorList, desconto_process)
            }

            let pacote_process = await _repo.pocessa_pacote(token_do_processo, _conn)
            this.checkRetorno(errorList, pacote_process)

            cargas_processadas = await _repo.get(_conn)
        } catch (e) {
            cargas_processadas = []
        }

        _repo.closeConnection(_conn)

        let _retorno = new RetornoModel(
            token_do_processo,
            errorList,
            cargas_processadas
        )

        return _retorno
    }
}
